/*
    defs.h
 
  Copyright (c) 2009-2014  Thomas Mueller

    This file is part of DssBH.
 
    DssBH is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DssBH is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DssBH.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!  \file   defs.h
     \brief  Global definitions for DssBH.
  
             If you have a big screen (1600x1200), then use 
             "DEFINES += USE_BIG_SCREEN" in the viewer.pro file.

     \mainpage  DssBH

     <H2>Introduction></H2>
     <p></p>

     <H2>DssBH License</H2>
     <p>Copyright &copy; 2009 by Thomas M&uuml;ller</p><br>
     <p>Permission to use, copy, modify, and distribute this software and its documentation und er the terms of the GNU General Public License is hereby granted. No representations are made about the suitability of this software for any purpose. It is provided "as is" without express  or implied warranty. See the <a href="http://www.gnu.org/copyleft/gpl.html">GNU General Publi
c License</a> for more details. </p>

     <br>
     <H2>Contact</H2>
     University of Stuttgart<br>Visualization Research Center (VISUS)<br>Nobelstrasse 15<br>70569 Stuttgart, Germany<br>
     Thomas.Mueller@vis.uni-stuttgart.de
    <br>

*/

#ifndef DEFS_H
#define DEFS_H



#ifdef _WIN32
#include <windows.h>
#endif

//#include <GL/gl.h>
//#include <GL/glu.h>


/* -----------------------------------
 *   global definitions
 * ----------------------------------- */
#ifndef DEG_TO_RAD
#define DEG_TO_RAD  0.017453292519943295770
#define RAD_TO_DEG  57.295779513082320875
#endif

#ifndef DBL_MAX
#define DBL_MAX 1.844674407370955616e19
#endif

#ifndef M_PI_2
#define M_PI_2 1.5707963
#endif

/* -----------------------------------
 *   GUI definitions
 * ----------------------------------- */

#ifdef  USE_BIG_SCREEN
#define    DEF_GUI_WIDTH     1600
#define    DEF_GUI_HEIGHT    1150
#define    DEF_DRAW_WIDTH    1560
#define    DEF_DRAW_HEIGHT    780
#define    DEF_TEXT_WIDTH     350
#define    DEF_WIDGET_HEIGHT   20
#define    DEF_DWIDGET_WIDTH   60
#else
#define    DEF_GUI_WIDTH     1270
#define    DEF_GUI_HEIGHT     740
#define    DEF_DRAW_WIDTH    1100
#define    DEF_DRAW_HEIGHT    550
#define    DEF_TEXT_WIDTH     300
#define    DEF_WIDGET_HEIGHT   20
#define    DEF_DWIDGET_WIDTH   60
#endif



/* -----------------------------------
 *   
 * ----------------------------------- */
#define  DEF_INIT_RS    2.0
#define  DEF_INIT_POS  10.0

#define  DEF_TIME_FACTOR 0.1

#define  DEF_FRAG_GAMMA  1.3
#define  DEF_FRAG_F0     0.5
#define  DEF_VERT_MPS    10

#define  DEF_MAX_MAGNITUDE  7.0


#define  DEF_HIPPARCOS_CATALOGUE_FILE_BIN  "data/hip.bin"

#define  DEF_DATA_SIGMA                "data/sigma.bin"
#define  DEF_DATA_PSITEMP              "data/psitemp.bin"
#define  DEF_DATA_ARRAY_OUTSIDE        "data/array_outside.bin"
#define  DEF_DATA_ARRAY_INSIDE         "data/array_inside.bin"

#if 1
#define  DEF_VERT_SHADER_NAME          "shader/bh_distort.vert.c"
#define  DEF_FRAG_SHADER_NAME          "shader/bh_distort.frag.c"
#else
#define  DEF_VERT_SHADER_NAME          "shader/standard.vert.c"
#define  DEF_FRAG_SHADER_NAME          "shader/standard.frag.c"
#endif

/* -----------------------------------------
 *   Hipparcos entries
 * ----------------------------------------- */
typedef struct struct_skyStarCat
{
  int    hip;
  float  ra;
  float  dec;
  float  plx;
  float  Vmag;
  float  bmv;
} skyStarCat_t;


/* -----------------------------------------
 *   Coefficients for temperature 
 *   calculation following Reed's paper.
 * ----------------------------------------- */
#define C1   -3.684
#define C2   14.551
#define C3    0.344
#define C4   -3.402
#define C5    8.037
#define C6   -8.499
#define C7   13.421
#define C8   -8.131
#define C9   -3.901
#define C10  -0.438

#define TEMPERATURE_ARRAY 11
#define MAGNITUDE_ARRAY   12

#endif
