/*
    dssbh_main.cpp

  Copyright (c) 2009-2014  Thomas Mueller

    This file is part of DssBH.

    DssBH is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DssBH is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DssBH.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!   \file  dssbh_main.cpp
      \brief Main file for GUI.

*/

#include <iostream>
#include <QApplication>
#include <QFile>
#include <QTextStream>

#include <src/dssbh_gui.h>

// ---------------------------------------------------
//    m a i n
// ---------------------------------------------------
int main( int argc, char *argv[] )
{
    QApplication app(argc, argv);

    QString cssStyle;
    QFile cssFile(":/viewer.css");
    if (cssFile.open(QFile::ReadOnly)) {
        QTextStream styleIn(&cssFile);
        while( !styleIn.atEnd()) {
            QString line = styleIn.readLine();
            if (!line.startsWith("#"))
                cssStyle += line;
        }
        cssFile.close();
    }
    else {
        fprintf(stderr,"Cannot read stylesheet!\n");
    }
    app.setStyleSheet(cssStyle);

    QLocale::setDefault(QLocale::C);
    QLocale::setDefault(QLocale(QLocale::English, QLocale::UnitedStates));

    double vmax = DEF_MAX_MAGNITUDE;
    if (argc==2)
        vmax = atof(argv[1]);
    DBH_View* view = new DBH_View(vmax);

    view->setWindowTitle( "Distort stellar sky by a Schwarzschild black hole" );
    view->show();

    app.exec();
    return 0;
}

