/*
    bh_distort.frag.c

  Copyright (c) 2009-2014  Thomas Mueller

    This file is part of DssBH.

    DssBH is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DssBH is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DssBH.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!   \file   bh_distort.frag.c
      \brief  Fragment shader for blackhole distortion.
*/
// -------------------------------------------------------------------------------

#version 330


#define  IlluminantC_x   0.3101
#define  IlluminantC_y   0.3162

#define  IlluminantD65_x 0.3127
#define  IlluminantD65_y 0.3291

#define  IlluminantE_x   0.33333
#define  IlluminantE_y   0.33333


#define  xRed    0.7355
#define  yRed    0.2645
#define  xGreen  0.2658
#define  yGreen  0.7243
#define  xBlue   0.1669
#define  yBlue   0.0085
#define  xWhite  IlluminantE_x
#define  yWhite  IlluminantE_y

#define minTemp   3.0


uniform sampler2D texSigma;
uniform sampler2D texPsiTemp;

uniform float  def_gamma;
uniform float  def_f0;


in float appMag;
in float logTempObs;
in float logTempStar;
in float one_over_mu;

layout(location = 0) out vec4 frag_color;

// -----------------------------------------------
//        xyz_to_rgb
// -----------------------------------------------
void  xyz_to_rgb( in vec3 xc, out vec3 rgb )
{
    vec3 xx = vec3( xRed, xGreen, xBlue );
    vec3 yy = vec3( yRed, yGreen, yBlue );
    vec3 zz = vec3(1.0) - xx - yy;

    vec3 w = vec3( xWhite, yWhite, 1.0-xWhite-yWhite );
    vec3 rgbx, rgby, rgbz, rgbw;

    /* xyz -> rgb matric, before scaling to white */
    rgbx = yy.gbr*zz.brg - yy.brg*zz.gbr;
    rgby = xx.brg*zz.gbr - xx.gbr*zz.brg;
    rgbz = xx.gbr*yy.brg - xx.brg*yy.gbr;

    /* White scaling factors. Dividing by w.y scales the white luminance to unity, as conventional. */
    rgbw = (rgbx*w.x + rgby*w.y + rgbz*w.z)/w.y;

    /* xyz -> rgb matrix, correctly scaled to white. */
    rgbx = rgbx/rgbw;
    rgby = rgby/rgbw;
    rgbz = rgbz/rgbw;

    rgb = rgbx*xc.x + rgby*xc.y + rgbz*xc.z;

    rgb.r = pow( rgb.r, 1.0/def_gamma );
    rgb.g = pow( rgb.g, 1.0/def_gamma );
    rgb.b = pow( rgb.b, 1.0/def_gamma );
}



// -----------------------------------------------
//        m a i n
// -----------------------------------------------
void main()
{
    vec4 color = vec4(vec3(0.0),1.0);

    float dist = length(gl_PointCoord - vec2(0.5));
    if (dist>0.5)     discard;

    float  tempPosObs  = clamp((logTempObs - minTemp)/1.7,0.0,0.9999);
    float  tempPosStar = clamp((logTempStar - minTemp)/1.7,0.0,0.9999);
    vec3 XYZ   = texture2D( texSigma, vec2(dist,tempPosObs) ).xyz;
    float psiVobs  = texture2D( texPsiTemp, vec2(tempPosObs,0)  ).r;
    float psiVstar = texture2D( texPsiTemp, vec2(tempPosStar,0) ).r;

#if 1
    float deltaOmegaUnlensed = pow(10.0,-0.4*appMag)/psiVstar*def_f0;
    vec3 xyz = XYZ*deltaOmegaUnlensed/one_over_mu*10.0;//*20.0; //*log(psiVobs)/log(10.0);
#else
    vec3 xyz = XYZ/(XYZ.x + XYZ.y + XYZ.z);
    float maxY = max(xyz.x,max(xyz.y,xyz.z));
    xyz = xyz/maxY;
#endif

    vec3 rgb;
    xyz_to_rgb( xyz, rgb );

    color = vec4( rgb, 1.0 );
    //  color = vec4(vec3(0.1/one_over_mu),1.0);

    //color = vec4(1,0,0,1);
    frag_color  = color;
}

