/*
    dssbh_gui.h

  Copyright (c) 2009-2014  Thomas Mueller

    This file is part of DssBH.

    DssBH is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DssBH is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DssBH.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!   \class  DBH_GUI
      \brief  GUI for DssBH.
*/


#ifndef  DBH_VIEW_H
#define  DBH_VIEW_H

#include <iostream>
#include <fstream>

#include <QDoubleSpinBox>
#include <QLabel>
#include <QLineEdit>
#include <QMainWindow>
#include <QMenu>
#include <QPushButton>
#include <QRadioButton>
#include <QSlider>
#include <QTimer>

#include <dssbh_render.h>


// ---------------------------------------------------
//    class definition:   DBH_View
// ---------------------------------------------------
class DBH_View : public QMainWindow
{
    Q_OBJECT

public:
    DBH_View ( double vmax = DEF_MAX_MAGNITUDE, QMainWindow* parent = 0 );
    ~DBH_View ( );

    // --------- public methods -----------
public:
    void    setVmagMax        ( double vmax );


    // --------- public slots -----------
public slots:
    void  slot_reset          ( );
    void  slot_quit           ( );
    void  slot_about          ( );

    void  slot_setInitValues    ( );
    void  slot_change_mode      ( );
    void  slot_update_crashTime ( );

    void  slot_update_currTime  ( );
    void  slot_update_currPos   ( );

    void  slot_setShaderParams  ( );

    void  slot_startstop        ( bool anim );
    void  slot_restart          ( );
    void  slot_animate          ( );
    void  slot_changeFactor     ( double val );


    // --------- protected methods -----------
protected:
    void  init         ( );
    void  initElements ( );
    void  initGUI      ( );
    void  initControl  ( );
    void  initActions  ( );
    void  initMenus    ( );


    // -------- private attributes --------
private:
    DBH_Render*  mRender;
    double       mCrashTime;
    double       mPosition;
    double       mVmagMax;


    /* ---- View menu ---- */
    QMenu*    mViewMenu;
    QAction*  mActionQuit;
    QAction*  mActionReloadShader;
    QAction*  mActionReset;

    /* ---- Help menu ---- */
    QMenu*    mHelpMenu;
    QAction*  mActionAbout;


    /* ---- Observer ---- */
    QLabel*         lab_initPosition;
    QDoubleSpinBox* spb_initPosition;
    QLabel*         lab_obsMode;
    QRadioButton*   rab_obsStatic;
    QRadioButton*   rab_obsFree;


    /* ---- Position/Time ---- */
    QLabel*         lab_crashTime;
    QLabel*         lab_crashTimeUnit;
    QLineEdit*      led_crashTime;
    QLabel*         lab_currPosition;
    QLabel*         lab_currPositionUnit;
    QSlider*        sli_currPosition;
    QLineEdit*      led_currPosition;
    QLabel*         lab_currTime;
    QLabel*         lab_currTimeUnit;
    QSlider*        sli_currTime;
    QLineEdit*      led_currTime;


    /* ---- Animate ---- */
    QPushButton*    pub_play;
    QPushButton*    pub_reset;
    QLabel*         lab_anim_factor;
    QDoubleSpinBox* spb_anim_factor;
    QTimer*         mTimer;
    double         mCurrTime;
    double         mTimeFactor;

    /* ---- Shader ---- */
    QLabel*         lab_frag_gamma;
    QDoubleSpinBox* spb_frag_gamma;
    QLabel*         lab_frag_f0;
    QDoubleSpinBox* spb_frag_f0;
    QLabel*         lab_vert_maxps;
    QSpinBox*       spb_vert_maxps;
};

#endif

