/*
    dssbh_render.h

  Copyright (c) 2009-2014  Thomas Mueller

    This file is part of DssBH.

    DssBH is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DssBH is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DssBH.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!   \class  DBH_Render
      \brief  OpenGL model for DssBH.
*/


#ifndef DSSBH_RENDER_H
#define DSSBH_RENDER_H

#include <iostream>
#include <fstream>
#include <cmath>

#include "GL3/gl3w.h"
#include <glm/glm.hpp>
#include <glm/gtc/type_ptr.hpp>
#include <glm/gtc/matrix_transform.hpp>

#include <defs.h>
#include <dssbh_utils.h>

#include <QGLWidget>


// ---------------------------------------------------
//    class definition:  
// ---------------------------------------------------
class DBH_Render : public QGLWidget
{
    Q_OBJECT

public:
    DBH_Render ( QGLFormat format, QWidget* parent = 0 );
    virtual ~DBH_Render ( );

    // --------- public methods -----------
public:
    void    setVmagMax   ( double vmax );
    bool    readHipBin   ( std::string filename );

    double  calc_vel     ( );
    bool    calc_ksiCrit ( const double xObs, const double x0, double &ksiCrit );

    double  getInitPosition   ( );
    void    setCurrPosition   ( double pos );
    double  calc_tauCrash     ( );

    void    setShaderParams   ( double gamma, double f0, int mps );

    // ---------- public slots ------------
public slots:
    void   slot_setInitValues  ( );
    void   slot_reloadShaders  ( );

    void   slot_setInitPosition ( double pos );
    void   slot_resetFragParams ( );



    // -------- protected methods ---------
protected:
    virtual void initializeGL      ( );
    virtual void initShaders       ( );
    virtual void initTextures      ( );
    virtual void paintGL           ( );
    virtual void drawPoints        ( );

    virtual void keyPressEvent     ( QKeyEvent* event );
    virtual void keyReleaseEvent   ( QKeyEvent* event );

    virtual void mousePressEvent   ( QMouseEvent* event );
    virtual void mouseReleaseEvent ( QMouseEvent* event );
    virtual void mouseMoveEvent    ( QMouseEvent* event );

    void    setInitValues        ( );
    void    recalcMatrix         ( );
    float   calc_temp            ( const float bmv );

    bool    readBinaryWithHeader ( std::string filename, GLuint* texID,  int &resX, int &resY, int &numChannels );

    void    setInitFragParams    ( );

    // -------- private attributes --------
private:

    int              mKeyPressed;
    Qt::MouseButton  mButtonPressed;

    double           mVmagMax;
    skyStarCat_t*    mSkyCat;


    std::string    mSigmaName;
    std::string    mPsiTempName;
    std::string    mKsiArrayOutsideName;
    std::string    mKsiArrayInsideName;
    std::string    mVertShaderName;
    std::string    mFragShaderName;


    GLuint         mTexID_sigma;
    GLuint         mTexID_psi;
    GLuint         mTexID_outside;
    GLuint         mTexID_inside;
    GLuint         mProgram;

    GLfloat*       mVerts;
    GLfloat*       mMagnitudes;
    GLfloat*       mTemperatures;
    int            mNumStars;
    GLuint         va,vbo,mbo,tbo;

    GLfloat        mDefGamma;
    GLfloat        mDefF0;
    GLuint         mDefMPS;

    double rs;
    double rObsInit;
    double rObsCurr;

    double x0;
    double mKsiCrit;

    GLfloat  rotationAngle1, oldAngle1;
    GLfloat  rotationAngle2, oldAngle2;
    GLfloat  rotation_matrix[16];
    GLfloat  rotDir;

    QPoint   last_pos;
    glm::mat4  mMVPmatrix;
};

#endif



